/******************************************************************************
 * $Id$
 * $Project maintainers$
 * $Revision$
 * $Tree$
 * MP2888 tool
 * Provided as is.
 * Debian Linux | GCC
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>
#include <time.h>
#include <math.h>
#include <wiringPi.h>
#include <wiringPiI2C.h>

#define I2C_MP2888_VCORE  0x0D
#define I2C_MP2888_VMEM   0x1C

static int mvid_ir3595 (void);
static int mvid_ir3595 (void);
static uint8_t print_help(void);
float lin5_11ToFloat(uint16_t wordValue);

// convert a LinearFloat5_11 formatted word into a floating point value
float lin5_11ToFloat(uint16_t wordValue)
{
    int8_t exponent = wordValue>>11; // extract exponent as MS 5 bits
    int16_t mantissa = wordValue & 0x7ff; // extract mantissa as LS 11 bits
    if( exponent > 0x0F ) exponent |= 0xE0; // sign extend exponent
    if( mantissa > 0x03FF ) mantissa |= 0xF800; // sign extend mantissa
    return mantissa * pow(2,exponent); // compute value as  mantissa * 2^(exponent)
}

const char menu[] =
    "\033[34;1m 0: Print this help \033[39;0m\r\n"
    "\033[31;1m 1: Set GPU voltage\r\n"
    " 2: Set MEM voltage\r\n"
    "\033[31;1m 3: Set GPU voltage ofs\r\n"
    "\033[33;1m 5: Read monitoring data\r\n"
    "\033[35;1m 6: Read temperature\033[39;0m\r\n"
    "\033[39;1m 9: Exit\r\n";
//    "7: Write ROM\r\n"
//    "8: Read ROM\r\n"

int mvid_ir3595(void) {
    int addr = wiringPiI2CSetup(I2C_MP2888_VCORE);
    int a = 0, i = 0;
    int phase_cnt = 0, phase_cnt_l2 = 0;
    short mvid_l1 = 0;
    unsigned int buffer[256];
    unsigned int prev[2];
    unsigned int magic = 0x7C25;

    printf("\033[2J\033[4;0H");

    buffer[0x27] = wiringPiI2CReadReg8(addr,0x27);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x28] = wiringPiI2CReadReg8(addr,0x28);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x29] = wiringPiI2CReadReg8(addr,0x29);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    
    printf ("Titan V device 0x0D detected: [25]%02X [88]%02X %02X\n", buffer[0x27], buffer[0x28], buffer[0x29]);
    
    while(1) {
        buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vcore : %d mVDC\n", buffer[0x8B]);
        buffer[0x88] = wiringPiI2CReadReg16(addr,0x88);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vin : %f VDC\n", ((float)buffer[0x88]) * 0.03125 );
        
        buffer[0x23] = wiringPiI2CReadReg16(addr,0x23);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("VOUT_OFFSET : %X \n", buffer[0x23]);
    
        printf("\033[12;0H[Enter \"0\" to exit]");

        printf("\033[7;0H\033[22;32mEnter VGPU voltage, mV :     \033[7;35H\033[1;32m");
        scanf("%d", &mvid_l1);

        if (mvid_l1 == 0) {
            return 0;
        }


	if (mvid_l1 == 1) {
    	    printf("\033[14;0H\033[22;32mReset to PWMVID");
                
    	    buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
    	    nanosleep((struct timespec[]){{0,200000}}, NULL);
    	    buffer[0xE1] = wiringPiI2CWriteReg16(addr,0x23,0xC2EA);
    	    nanosleep((struct timespec[]){{0,200000}}, NULL);
	    return 0;
	}

        buffer[0x21] = (uint16_t)((float)mvid_l1 / 6.25) ;
	printf("\033[9;0H\033[22;32mCode : %X", buffer[0x21]);
                
        buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0xE1] = wiringPiI2CWriteReg16(addr,0xE1,0xC2FA);
        nanosleep((struct timespec[]){{0,200000}}, NULL);

//        buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
//        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0x23] = wiringPiI2CWriteReg16(addr,0x21,buffer[0x21]);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0x23] = wiringPiI2CReadReg16(addr,0x23);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("VOUT_OFFSET : %X", buffer[0x23]);
        
        //buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        //nanosleep((struct timespec[]){{0,200000}}, NULL);
        //printf ("   VcoreOfs : %d mVDC, password = %X\n", buffer[0x8B], password);
        
        //magic++;
        
  //      if (magic == 0x7B35) { return 1; };
    };
}

int mvid_ir3595_ofs(void) {
    int addr = wiringPiI2CSetup(I2C_MP2888_VCORE);
    int a = 0, i = 0;
    int phase_cnt = 0, phase_cnt_l2 = 0;
    short mvid_l1 = 0;
    unsigned int buffer[256];
    unsigned int prev[2];
    unsigned int magic = 0x7C25;

    printf("\033[2J\033[4;0H");

    buffer[0x27] = wiringPiI2CReadReg8(addr,0x27);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x28] = wiringPiI2CReadReg8(addr,0x28);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x29] = wiringPiI2CReadReg8(addr,0x29);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    
    printf ("Titan V device 0x0D detected: [25]%02X [88]%02X %02X\n", buffer[0x27], buffer[0x28], buffer[0x29]);
    
    while(1) {
        buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vcore : %d mVDC\n", buffer[0x8B]);
        buffer[0x88] = wiringPiI2CReadReg16(addr,0x88);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vin : %f VDC\n", ((float)buffer[0x88]) * 0.03125 );
        
        buffer[0x23] = wiringPiI2CReadReg16(addr,0x23);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("VOUT_OFFSET : %X \n", buffer[0x23]);
    
        printf("\033[12;0H[Enter \"0\" to exit]");

        printf("\033[7;0H\033[22;32mEnter VGPU offset, mV :     \033[7;35H\033[1;32m");
        scanf("%d", &mvid_l1);

        if (mvid_l1 == 0) {
            return 0;
        }


	if (mvid_l1 == 1) {
    	    printf("\033[14;0H\033[22;32mReset to PWMVID");
                
    	    buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
    	    nanosleep((struct timespec[]){{0,200000}}, NULL);
    	    buffer[0xE1] = wiringPiI2CWriteReg16(addr,0x23,0xC2EA);
    	    nanosleep((struct timespec[]){{0,200000}}, NULL);
	    return 0;
	}

        buffer[0x23] = (uint16_t)mvid_l1 ;
	printf("\033[9;0H\033[22;32mCode : %X", buffer[0x21]);
                
        buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
//        buffer[0xE1] = wiringPiI2CWriteReg16(addr,0xE1,0xC2FA);
//        nanosleep((struct timespec[]){{0,200000}}, NULL);

//        buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
//        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0x23] = wiringPiI2CWriteReg16(addr,0x23,buffer[0x23]);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0x23] = wiringPiI2CReadReg16(addr,0x23);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("VOUT_OFFSET : %X", buffer[0x23]);
        
        //buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        //nanosleep((struct timespec[]){{0,200000}}, NULL);
        //printf ("   VcoreOfs : %d mVDC, password = %X\n", buffer[0x8B], password);
        
        //magic++;
        
  //      if (magic == 0x7B35) { return 1; };
    };
}

int mvid_ir3570(void) {
    int addr = wiringPiI2CSetup(I2C_MP2888_VMEM);
    int a = 0, i = 0;
    int phase_cnt = 0, phase_cnt_l2 = 0;
    short mvid_l1 = 0;
    unsigned int buffer[256];
    unsigned int prev[2];
    unsigned int magic = 0x7B34; // VC : 7C25

    printf("\033[2J\033[4;0H");

    buffer[0x27] = wiringPiI2CReadReg8(addr,0x27);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x28] = wiringPiI2CReadReg8(addr,0x28);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x29] = wiringPiI2CReadReg8(addr,0x29);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    
    printf ("Titan V device 0x0D detected: [25]%02X [88]%02X %02X\n", buffer[0x27], buffer[0x28], buffer[0x29]);
    
    while(magic != 0xFFFF) {
        buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vcore : %d mVDC\n", buffer[0x8B]);
        buffer[0x88] = wiringPiI2CReadReg16(addr,0x88);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("Titan V Vin : %f VDC\n", ((float)buffer[0x88]) * 0.03125 );
        
        printf("\033[12;0H[Enter \"0\" to exit]");

        printf("\033[7;0H\033[22;32mEnter VGPU offset voltage, bit :     \033[7;35H\033[1;32m");
        scanf("%d", &mvid_l1);

        if (mvid_l1 == 0) {
            return 0;
        }

        buffer[0x23] = mvid_l1;
        
        buffer[0xF2] = wiringPiI2CWriteReg16(addr,0xF2,magic);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
                
        buffer[0x23] = wiringPiI2CWriteReg16(addr,0x23,buffer[0x23]);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        
        buffer[0x23] = wiringPiI2CReadReg16(addr,0x23);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("VOUT_OFFSET : %X", buffer[0x23]);
        
        buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("   VcoreOfs : %d mVDC, password = %X\n", buffer[0x8B], magic);
        
        //magic++;
        
  //      if (magic == 0x7B35) { return 1; };
    };
}

int read_hwm(void) {
    int addr = wiringPiI2CSetup(I2C_MP2888_VCORE);
    int addrm = wiringPiI2CSetup(I2C_MP2888_VMEM);
    int a = 0, i = 0;
    int phase_cnt = 0, phase_cnt_l2 = 0;
    short mvid_l1 = 0;
    unsigned int buffer[256];
    unsigned int prev[2];
    unsigned int magic = 0x7C25;

    printf("\033[2J\033[4;0H");

    buffer[0x27] = wiringPiI2CReadReg8(addr,0x27);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x28] = wiringPiI2CReadReg8(addr,0x28);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    buffer[0x29] = wiringPiI2CReadReg8(addr,0x29);
    nanosleep((struct timespec[]){{0,200000}}, NULL);
    
    printf ("Titan V device 0x0D detected: [25]%02X [88]%02X %02X\n", buffer[0x27], buffer[0x28], buffer[0x29]);
    
    while(1) {
        buffer[0x8B] = wiringPiI2CReadReg16(addr,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("\033[1;31mVcore ; %05d mVDC;", buffer[0x8B]);
        buffer[0x8C] = wiringPiI2CReadReg16(addr,0x8C);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("\033[1;32mIc ; %05d ADC;", (buffer[0x8C] & 0x0FFF) / 2);
        buffer[0x96] = wiringPiI2CReadReg16(addr,0x96);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("\033[1;34mPc ; %05d W;", buffer[0x96]);

        buffer[0x1B] = wiringPiI2CReadReg16(addrm,0x8B);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("\033[1;36mVmem ; %04d mVDC ", buffer[0x1B] );
        buffer[0x88] = wiringPiI2CReadReg16(addr,0x88);
        nanosleep((struct timespec[]){{0,200000}}, NULL);
        printf ("\033[1;35m Vin ; %.2f VDC; \033[0;39m\n", ((float)(buffer[0x88] & 0x03FF)) * 0.03125);

        nanosleep((struct timespec[]){{0,250000000}}, NULL);

    };
}


int main()
{
    int choice = 0;

    while(1) {

    switch (choice) {
    choice = print_help();
//    printf ("%d X", choice);
            case 0: { choice = print_help();break; }
            case 1: { choice = 0;mvid_ir3595();break; }
            case 2: { choice = 0;mvid_ir3570();break; }
            case 3: { choice = 0;mvid_ir3595_ofs();break; }
            case 5: { choice = 0;read_hwm();break; }
            //case 6: { choice = 0;read_gput();break; }
            case 9: { printf ("\033[2J\033[0;0H");return 0;break; }
            default: { choice = 0; printf ("\033[2;0H Invalid, try again");break;}
        };
    };

    return 0;
}


uint8_t print_help(void) {
    int menuitem = 255;
    printf ("\033[2J\033[0;0H\033[22;41;37;1m ..:*** DC/DC Tuner v0.2 ***:.. \
 .:  PMBus/SMBus master debug :. Titan V version \033[0;40m");
/*    printf ("\033[22;41;37;1m ..::*** PMBus ECU v0.2 ***::.. \
 .:  Raspberry Pi version   :.  \r\n\033[0;40m");*/

    printf ("\033[3;0H\n%s", menu);
    printf ("\r\n\n\033[22;33m Enter menu item: \033[0m");
    scanf("%d", &menuitem);
    return menuitem;
}
